#include <p32xxxx.h>

// 外部の8MHz水晶発振器から40MHzのクロックを生成するための設定
#pragma config FNOSC = PRIPLL // 発信源 = 主外部発振器
#pragma config POSCMOD = XT // 主発振の方法　= 外部高精度発振器を使う
#pragma config FPLLIDIV = DIV_2 // PLL入力分数 = x1/2
#pragma config FPLLMUL = MUL_20 // PLL逓倍比 = x20
#pragma config FPLLODIV = DIV_2 // PLL出力分数 = x1/2
#pragma config FPBDIV = DIV_1 //周辺モジュールクロック倍数 : x1/1
#pragma config FWDTEN = OFF
#pragma config ICESEL = ICS_PGx2 //ICE接続チャンネル = 2

unsigned char toggle = 0, count = 0;

// 割り込みレベル4、4番ベクタ(Timer1に該当)
void __attribute__((interrupt(IPL4AUTO))) __attribute__((vector(4))) OnTimer1(void)
{
    IFS0bits.T1IF = 0; // 割り込みフラグをリセット

    count++;
    if(count >= 10){
        toggle = !toggle;
        count = 0;
        PORTBbits.RB0 = toggle; // RB0のスイッチを切り替える      
    }
}

void initTimer1()
{
    T1CONbits.ON = 0;    // 設定のため、タイマー1を無効にする
    TMR1 = 0;            // タイマー1をリセット
     
    // 100ms周期で割り込み <- 250ns(1 / 40MHz) * 256 * 15625(0x3D09) = 100ms
    T1CONbits.TCKPS = 3; // プリスケーラ : x1/256
    PR1 = 0x3D09;        // カウンター
    
    INTCONbits.MVEC = 1; // マルチベクタ割り込みを有効にする
    
    IPC1bits.T1IP = 4;   // 割り込み優先度(0-7)
    IFS0bits.T1IF = 0;   // 割り込みフラグをリセット
    IEC0bits.T1IE = 1;   // タイマー1の割り込みを許可
    asm volatile("ei");  // 割り込み機能を使用する命令 : __builtin_enable_interrupts()と同義
    T1CONbits.ON = 1;    // タイマー1有効
}

int main(void)
{
    TRISBbits.TRISB0 = 0; // B0を出力にする
    PORTBbits.RB0 = 0;    // B0 = off
     
    initTimer1();
    while(1) ;
}